/* 
**  directives.c
*/ 

#include "mod_mp3.h"

MP3_EXPORT(const char *) add_mp3(cmd_parms * cmd, void *mconfig, char *param) {
	mp3_conf *cfg = (mp3_conf *) mconfig;
	char *temp=NULL;
	struct stat sbuf;
	DIR *dir;
	struct dirent *file = NULL;
	mp3_data *bank = NULL;
	array_header *directories;
	char **directory = NULL;
	int x = 0;

	if(cfg->files == NULL)
		cfg->files = ap_make_array (cmd->pool, 5, sizeof (mp3_data *));

	/* Silent error, bad thing */
	if(stat(param, &sbuf))
		return NULL;


	if(S_ISDIR(sbuf.st_mode)) {
		if(cfg->load_directories == NULL)
			cfg->load_directories = ap_make_array (cmd->pool, 5, sizeof (char *));
		*(char **) ap_push_array (cfg->load_directories) = ap_pstrdup (cmd->pool, param);

		if(!cfg->reload_request) 
			if(load_directory(cmd->pool, cfg->files, param, cfg->cache_enabled) != OK) {
				ap_log_error (APLOG_MARK, APLOG_ERR, cmd->server,
					"Could not load the directory: %s(%s)", param, strerror(errno));
			}
	} else {
		if(cfg->load_files == NULL)
			cfg->load_files = ap_make_array (cmd->pool, 5, sizeof (char *));
		*(char **) ap_push_array (cfg->load_files) = ap_pstrdup (cmd->pool, param);

		if(!cfg->reload_request) 
			if(load_file(cmd->pool, cfg->files, param, param, cfg->cache_enabled) != OK) {
				ap_log_error (APLOG_MARK, APLOG_ERR, cmd->server,
					"Could not load the file: %s(%s)", param, strerror(errno));
			}
	}


	return NULL;
}

MP3_EXPORT(const char *) add_mp3_playlist(cmd_parms * cmd, void *mconfig, char *param) {
	mp3_conf *cfg = (mp3_conf *) mconfig;

	if(cfg->files == NULL)
		cfg->files = ap_make_array (cmd->pool, 5, sizeof (mp3_data *));

	if(cfg->load_playlists == NULL)
		cfg->load_playlists = ap_make_array (cmd->pool, 5, sizeof (char *));
	*(char **) ap_push_array (cfg->load_playlists) = ap_pstrdup (cmd->pool, param);

	if(!cfg->reload_request) 
		if(load_playlist(cmd->pool, cfg->files, param, cfg->cache_enabled) != OK) {
			ap_log_error (APLOG_MARK, APLOG_ERR, cmd->server,
				"Could not load the playlist: %s(%s)", param, strerror(errno));
		};

	return NULL;
}

MP3_EXPORT(const char *) add_local_stream(cmd_parms * cmd, void *mconfig, char *name) {
	mp3_conf *cfg = (mp3_conf *) mconfig;
	struct stat sbuf;
	FILE *file;
	cfg->local_stream = ap_pstrcat(cmd->pool, "/tmp/mod_mp3.", name, NULL);
	
	if(!stat(cfg->local_stream, &sbuf))
		unlink(cfg->local_stream);

	if(mkfifo(cfg->local_stream, 0664) == -1) {
		ap_log_error (APLOG_MARK, APLOG_ERR, cmd->server,
			"Could not create stream: %s(%s)", name, strerror(errno));
	}

	return NULL;
}

MP3_EXPORT(const char *) enable_request_reload(cmd_parms * cmd, void *mconfig, int flag) {
	mp3_conf *cfg = (mp3_conf *) mconfig;
	if(cfg->cache_enabled) {
		ap_log_error (APLOG_MARK, APLOG_ERR, cmd->server,
			"You can not turn on MP3ReloadRequest if you have the cache enabled.");
		exit(1);
	}

	cfg->reload_request = 1;
	return NULL;
}

MP3_EXPORT(const char *) enable_cache(cmd_parms * cmd, void *mconfig, int flag) {
	mp3_conf *cfg = (mp3_conf *) mconfig;

	if(cfg->files) {
		ap_log_error (APLOG_MARK, APLOG_ERR, cmd->server,
			"You can not turn on Cache in your httpd.conf after you have specified MP3 or MP3Playlist");
		exit(1);
	}
	if(cfg->reload_request) {
		ap_log_error (APLOG_MARK, APLOG_ERR, cmd->server,
			"You can not turn on Cache if you have enabled MP3ReloadRequest");
		exit(1);
	}

	cfg->cache_enabled = 1;

	return NULL;
}
