/* 
**  load.c
*/ 

#include "mod_mp3.h"

/* This is where we chew up a lot of shared memory */
mp3_data * mp3_create_content(pool *p, char *filename, char *name, int flag) {
	mp3_data *bank;
	int x = 0;
	struct stat buf;
	FILE *file;
	int fd = -1;
	caddr_t temp;
	id3 *p_id3 = NULL;

	if(stat(filename, &buf)) {
		return NULL;
	}
	ap_no2slash(filename);

	x = sizeof(id3);
	bank = ap_pcalloc (p, sizeof (mp3_data));
	p_id3 = &bank->id;
	bank->filename = ap_pstrdup(p, filename);
	bank->size = buf.st_size;
	bank->data = NULL;
	bank->time = buf.st_mtime;
	bank->signature = (const char *)ap_md5(p, filename);
	memset(p_id3, 0, x);

	/* Yes, this is redundanct at the moment */
	/* Ok, long run we need to check this */
	fd = open(filename, O_RDONLY, 0);
	/* Need to come back and look at this later, since this will eat up memory */
	if (lseek(fd,-128,SEEK_END)>0){
		if (read(fd,p_id3,x)==x){
			if(!strncmp(p_id3->tag, "TAG", 3)) {
				bank->name = ap_pstrndup(p, p_id3->songname,30); 
#ifdef DEBUG
				printf("ID3 name on :%s:\n", p_id3->songname); 
#endif
			}
		} 
		if(!bank->name)
			bank->name = ap_pstrdup(p, name);
	}
#ifdef DEBUG
	printf("Adding %s\n", bank->name);
#endif
	if(flag) {
		lseek(fd, 0, SEEK_SET);
		temp = mmap(NULL, buf.st_size, PROT_READ, MAP_SHARED, fd, 0);
		if(temp ==(caddr_t)-1)
			printf("Woops, blew up, %d\n", bank->data); 
		bank->data = temp;
	}
	close(fd);

	return bank;
}

MP3_EXPORT(int) load_playlist(pool *p, array_header *files, char *file, int cache_enabled) {
	FILE *file_ptr;
	char buf[HUGE_STRING_LEN];
	struct stat sbuf;
	mp3_data *bank = NULL;
	int x = 0;
	size_t length = 0;
	if (!(file_ptr = ap_pfopen (p, file, "r"))) {
		return HTTP_NOT_FOUND;
	}
	while (fgets (buf, sizeof (buf), file_ptr)) {
		length = strlen(buf);
		buf[length-1] = '\0';
		if(load_file(p, files, buf, buf, cache_enabled) != OK) {
			/* Need to add an error message here */
		} 
	}
	ap_pfclose (p, file_ptr);

	return OK;
}

MP3_EXPORT(int) load_file(pool *p, array_header *files, char *file, char *name, int cache_enabled) {
	mp3_data *bank = NULL;

	if(!( bank = mp3_create_content(p, file, name, cache_enabled))) {
		return HTTP_NOT_FOUND;
	}
	*(mp3_data **) ap_push_array (files) = (mp3_data *) bank;

	return OK;
}

MP3_EXPORT(int) load_directory(pool *p, array_header *files, char *param, int cache_enabled) {
	pool *temp_pool = NULL;
	char *temp=NULL;
	struct stat sbuf;
	DIR *dir;
	struct dirent *file = NULL;
	mp3_data *bank = NULL;
	array_header *directories;
	char **directory = NULL;
	int x = 0;

	temp_pool = ap_make_sub_pool(p);

	directories = ap_make_array (temp_pool, 15, sizeof (char *));
	*(char **) ap_push_array (directories) = ap_pstrdup (temp_pool, param);

	while ( x < directories->nelts) {
		directory = (char **)directories->elts;
		param = directory[x];
		x++;
		if (!(dir = ap_popendir(temp_pool, param))) {
			continue;
		} else {
			while(file = readdir(dir)) {
				if(stat((temp = ap_pstrcat(temp_pool, param, "/", file->d_name, NULL)), &sbuf) == 0){
/*************************************/
					if(S_ISREG(sbuf.st_mode)) {
/*************************************/
						if(load_file(p, files, temp, file->d_name, cache_enabled) != OK) {
							/* Need to add an error message here */
						} 
/*************************************/
					}
					if(S_ISDIR(sbuf.st_mode)) {
						if(file->d_name[0] != '.') {
#ifdef DEBUG
							printf("Stacking %s\n", temp);
#endif
							*(char **) ap_push_array (directories) = ap_pstrdup (temp_pool, temp);
						}
					}
/*************************************/
				} else {
								printf("Could not stat %s\n", temp);
				}
			}
			ap_pclosedir(temp_pool, dir);
		}
	}

	ap_destroy_pool(temp_pool);
	return OK;
}

MP3_EXPORT(array_header *) reload_files(server_rec *server, pool *p, mp3_conf *cfg) {
	array_header *files = ap_make_array (p, 5, sizeof (mp3_data *));
	int x = 0;
	char **names = NULL;

	if(cfg->load_directories) {
		names = (char **)cfg->load_directories->elts;
		for(x = 0; x < cfg->load_directories->nelts; x++) {
			if(load_directory(p, files, names[x], cfg->cache_enabled) != OK) {
				ap_log_error (APLOG_MARK, APLOG_ERR, server,
					"Could not load the directory: %s(%s)", names[x], strerror(errno));
			}
		}
	}

	if(cfg->load_playlists) {
		names = (char **)cfg->load_files->elts;
		for(x = 0; x < cfg->load_files->nelts; x++) {
			if(load_playlist(p, files, names[x], cfg->cache_enabled) != OK) {
				ap_log_error (APLOG_MARK, APLOG_ERR, server,
					"Could not load the playlist: %s(%s)", names[x], strerror(errno));
			};
		}
	}

	if(cfg->load_files) {
		names = (char **)cfg->load_files->elts;
		for(x = 0; x < cfg->load_files->nelts; x++) {
			if(load_file(p, files, names[x], names[x], cfg->cache_enabled) != OK) {
				ap_log_error (APLOG_MARK, APLOG_ERR, server,
					"Could not load the file: %s(%s)", names[x], strerror(errno));
			}
		}
	}

	return files;
}
