/* 
**  mod_mp3.c
*/ 

#include "mod_mp3.h"

/* Setup for our scoreboard */
static key_t shmkey = IPC_PRIVATE;

/* sends out the right headers for data requests */
void send_headers(request_rec *r, mp3_conf* cfg, request_data *request) {
	if(request->type == 1) {
		send_icecast_headers(r, cfg, request);
	} else if (request->type == 0){
		send_shout_headers(r, cfg, request);
	} else {
		r->content_type = "audio/mpeg";      
		ap_send_http_header(r);
	}

}

scoreboard * get_scoreboard(int shmid) {
	scoreboard *shm = NULL;

	shm = (scoreboard *)shmat(shmid, NULL, 0);
	if(shm == (scoreboard *)-1) 
		return NULL;

	return shm;
}

static void cleanup_scoreboard(void *data) {
	mp3_conf *cfg = (mp3_conf *)data;
	shmctl(cfg->shmid, IPC_RMID, 0);
}

/* Creates the per virtualhost, directory, location information */
static void * mconfig_for_directory(pool *p, char *dir) {
	mp3_conf *cfg = ap_pcalloc (p, sizeof (mp3_conf));
	int shmid = -1;
	scoreboard *shm = NULL;

	if ((shmid = shmget(shmkey, sizeof(scoreboard), IPC_CREAT | 0666)) < 0) {
		printf("%s - Cannot create Shared Memory", __FILE__);
		exit(1);
	}

	if(!(shm = get_scoreboard(shmid))) {
		printf("%s - Cannot attach to Shared Memory", __FILE__);
		exit(1);
	}

	ap_register_cleanup(p, cfg, cleanup_scoreboard, ap_null_cleanup);

	cfg->enabled = 0;
	cfg->random_enabled = 0;
	cfg->cache_enabled = 0;
	cfg->reload_request = 0;
	cfg->files = NULL;
	cfg->local_stream = NULL;
	cfg->cast_name = DEFAULT_CAST_NAME;
	cfg->genre_name = DEFAULT_GENRE_NAME;
	cfg->load_files = NULL;
	cfg->load_directories = NULL;
	cfg->load_playlists = NULL;
	cfg->shmid = shmid;
	shm->time = time(NULL);
	cfg->generation = time(NULL);

	return (void *) cfg;
}	

static int stream_content(request_rec *r, mp3_conf *cfg, mp3_data *content, request_data *request) {
	FILE *file;
	int b = 0;
	int x = 0;
	int latch = 0;
	unsigned char *temp = NULL;
	const char *message = NULL;

#ifdef DEBUG
		printf ("Playing %s \n", content->name);
#endif

	if(request->udp) {
		message = get_udp_message(r->pool, content->name, request->url, cfg->cast_name);
		send_udp_message(r, request->udp, message);
	}

	ap_hard_timeout("mod_mp3_write", r);

	if(content->data) {
		if(request->shout) {
/* Bug in this at the moment */
			temp = (unsigned char *)content->data;
			for(x = 0; x < content->size; x++) {
				if(shout_write(r, temp[x], content->name, request->url, &latch) == -1)
					return HTTP_REQUEST_TIME_OUT;
			}
		} else {
			x = ap_send_mmap(content->data, r, 0, content->size);
			if(!x)
				return HTTP_REQUEST_TIME_OUT;
		} 
	} else {
		if (!(file = ap_pfopen(r->pool, content->filename, "r"))) { 
			ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r, "File not found: %s(%s)", content->filename, strerror(errno));
			return OK;
		}

		while((b = fgetc(file)) != EOF){
			if(request->shout) {
				if(shout_write(r,b, content->name, request->url, &latch) == -1)
					return HTTP_REQUEST_TIME_OUT;
			} else {
				if(ap_rputc(b,r) == -1)
					return HTTP_REQUEST_TIME_OUT;
			}
		}
	}
	ap_kill_timeout(r);

	return OK;
}


static int mp3_selection_handler(request_rec *r) {
	mp3_conf *cfg;
	mp3_data **filenames = NULL;
	int x = 0;
	static char *url = NULL;
	cfg = ap_get_module_config(r->per_dir_config, &mp3_module);

	url = ap_psprintf(r->pool, "http://%s:%d%s?op=play&amp;song=", r->hostname, r->server->port, r->uri);
	

	filenames = (mp3_data **)cfg->files->elts;

	r->content_type = "text/html";      
	ap_rprintf(r, "%s<HTML><HEAD>\n<TITLE>%s</TITLE>\n</HEAD><BODY>\n", DOCTYPE_HTML_3_2, cfg->cast_name);
	ap_rprintf(r, "<FORM ACTION=\"http://%s:%d%s\" METHOD=\"GET\">\n", r->hostname, r->server->port, r->uri);
	
	for(x = 0; x < cfg->files->nelts; x++) {
		ap_rprintf(r, "<BR><INPUT TYPE=\"checkbox\" name=\"song\" value=\"%s\">  <A HREF=\"%s%s\">%s</A></BR>\n", filenames[x]->signature, url, filenames[x]->signature, filenames[x]->name);
	}
	ap_rputs("<INPUT TYPE=\"hidden\" NAME=\"op\" VALUE=\"song\">", r);
	ap_rputs("<INPUT TYPE=\"reset\" VALUE=\"Reset\">", r);
	ap_rputs("<INPUT TYPE=\"submit\" NAME=\"submit\" VALUE=\"Play\">", r);
	ap_rputs("</FORM>", r);

	return OK;
}

void print_channel(request_rec *r, mp3_conf *cfg) {
	char *time_string = NULL;

	time_string = ap_pstrdup(r->pool, ap_ht_time(r->pool, r->finfo.st_mtime, TIME_FORMAT, 0));
	ap_rprintf(r, "<title>%s</title>\n", cfg->cast_name);
	ap_rprintf(r, "<description>%s</description>\n", cfg->genre_name);
	ap_rprintf(r, "<pubDate>%s</pubDate>\n", time_string);
	ap_rprintf(r, "<lastBuildDate>%s</lastBuildDate>\n", time_string);
	ap_rprintf(r, "<webMaster>%s</webMaster>\n", r->server->server_admin);
	ap_rprintf(r, "<link>http://%s%s</link>\n\n", r->hostname, r->uri);
}

static int mp3_rss_handler(request_rec *r) {
	mp3_conf *cfg = ap_get_module_config (r->per_dir_config, &mp3_module);
	request_data *request = ap_get_module_config(r->request_config, &mp3_module);

	int x = 0;
	mp3_data **filenames = NULL;
	const char *pattern = ap_table_get(request->args, "pattern");

	r->content_type = "text/xml";      
	ap_send_http_header(r);

	if (r->header_only) {
		return OK;
	}

	ap_rputs(HEADER, r);
	print_channel(r, cfg);

	filenames = (mp3_data **)cfg->files->elts;

	for(x = 0; x < cfg->files->nelts; x++) {
		if(pattern)
			if(ap_fnmatch(pattern, filenames[x]->name, FNM_CASE_BLIND))
				continue;
	
		ap_rputs("<item>\n", r);
		ap_rprintf(r, "<title>%s</title>\n", ap_escape_html(r->pool, filenames[x]->name));
		ap_rprintf(r, "<link>http://%s:%d%s?op=play&amp;song=%s</link>\n", r->hostname, r->server->port, r->uri, filenames[x]->signature);
		ap_rputs("</item>\n", r);
		ap_rputs("\n", r);
	}

	/* So now we have a bunch of stuff in memory, lets sort */
	ap_rputs(FOOTER, r);

	return OK;
}

/* This creates the per request content info */
request_data * create_request(request_rec *r, mp3_conf *cfg) {
	request_data *request = NULL;
	const char *icy_string_header = NULL;
	const char *udp = NULL;

	request = ap_pcalloc (r->pool, sizeof (request_data));
	request->url = make_basename(r);
	request->op = NULL;
	request->order = NULL;
	request->command = NULL;
	request->args = NULL;
	request->udp = 0;
	request->shout = 0;
	request->type = -1;
	request->random = 0;
	request->files = cfg->files;

	/* This is where we decide what our client is */
	if(udp = ap_table_get(r->headers_in, "x-audiocast-udpport")) {
		request->udp = atoi((char *)udp);
	} 

	if(ap_table_get(r->headers_in, "Icy-MetaData")) {
		request->shout = 1;
	}

	if(request->shout) {
		request->type = 0;
	} else if (request->udp) {
		request->type = 1;
	} else {
		request->type = -1;
	}

	if(r->args) {
		request->args = parse_args(r);
		request->op = ap_table_get(request->args, "op");
		request->command = ap_table_get(request->args, "command");
		request->order = ap_table_get(request->args, "order");
	}

#ifdef CRAP
	/* Just to solve a bug */
	if(cfg->cache_enabled) {
		request->type = -1;
	}
	if(request->order && (!ap_fnmatch(request->order, "random", FNM_CASE_BLIND))) {
		request->random = 1;
	}
#endif

#ifdef DEBUG
	table_list("Args ", request->args);
#endif
	
	ap_set_module_config(r->request_config, &mp3_module, request);

	return request;
}

static int mp3_fixup(request_rec *r) {
	mp3_conf *cfg = ap_get_module_config(r->per_dir_config, &mp3_module);
	request_data *request = NULL;
	scoreboard *board = NULL;
	mp3_data **banks = NULL;
	array_header *files = NULL;
	array_header *songs = NULL;
	char **song = NULL;
	const char *pattern = NULL;
	int x,j =0;

	if(!cfg->enabled)
		return DECLINED;


	banks = (mp3_data **)cfg->files->elts;
	if(!(board = get_scoreboard(cfg->shmid))) {
		printf("Woops, no scoreboard file \n");
	}

	if(cfg->reload_request || (board->time > cfg->generation)) {
		cfg->files = reload_files(r->server, r->pool, cfg);
		cfg->generation = time(NULL);
	}

	request = create_request(r, cfg);
	if(request) {

		if(request->command) {
			if(!ap_fnmatch(request->command, "reload", FNM_CASE_BLIND)) {
				board->time = time(NULL);
				cfg->files = reload_files(r->server, r->pool, cfg);
				cfg->generation = time(NULL);
			}
		}

#ifdef DEBUG
		printf("OP:%s:\n", request->op);
#endif
		if(request->op) {
			if(songs = get_songs(r->pool, request->args)) {
				array_header *files = ap_make_array (r->pool, 5, sizeof (mp3_data *));
				songs = get_songs(r->pool, request->args);
				song = (char **)songs->elts;
				for(j = 0; j < songs->nelts; j++) {
					for(x = 0; x < cfg->files->nelts; x++) {
						if(!strcmp(song[j], banks[x]->signature)) {
							*(mp3_data **) ap_push_array (files) = (mp3_data *) banks[x];
						}
					}
				}
				request->files = files;
			}

			if(pattern = ap_table_get(request->args, "pattern")) {
				array_header *files = ap_make_array (r->pool, 5, sizeof (mp3_data *));
				for(x = 0; x < cfg->files->nelts; x++) {
					if(!ap_fnmatch(pattern, banks[x]->name, FNM_CASE_BLIND)) {
						*(mp3_data **) ap_push_array (files) = (mp3_data *) banks[x];
					}
				}
				request->files = files;
			}

			if(!ap_fnmatch(request->op, "select", FNM_CASE_BLIND)) {
				r->handler = "mp3-selection";
			} else if(!ap_fnmatch(request->op, "rss", FNM_CASE_BLIND)) {
				r->handler = "mp3-rss";
			} else if(!ap_fnmatch(request->op, "list", FNM_CASE_BLIND)) {
					r->handler = "mp3-rss";
			} else if(!ap_fnmatch(request->op, "play", FNM_CASE_BLIND)) {
				r->handler = "mp3";
			}

			
			return DECLINED;
		}
	} 

	if(cfg->random_enabled) {
		r->handler = "mp3-random";
	} else {
		r->handler = "mp3";
	}

	if(cfg->local_stream) {
		if(!strcmp("mp3", r->handler)){
			r->handler = "mp3-stream";
		}
	}

	return DECLINED;
}

static void mp3_child(server_rec *s, pool *p) {
//	mp3_conf *cfg = ap_get_module_config (r->per_dir_config, &mp3_module);
}

static int mp3_handler(request_rec *r) {
	int x = 0;
	mp3_data **banks = NULL;
	mp3_conf *cfg = ap_get_module_config (r->per_dir_config, &mp3_module);
	request_data *request = ap_get_module_config(r->request_config, &mp3_module);
	/* for better PRNG seeding with microseconds versus current seconds */
	struct timeval tv_struct[2];
	gettimeofday(tv_struct, 0);
	srandom (tv_struct[0].tv_sec % tv_struct[0].tv_usec);
 

#ifdef DEBUG
	table_list("HEADERS ", r->headers_in);
#endif
	banks = (mp3_data **)request->files->elts;

	send_headers(r, cfg, request);

	if(request->random) {
		for(;;) {
			x = ((int)random() % request->files->nelts);
			if(stream_content(r, cfg, banks[x], request) != OK)
				return OK;
		}
	} else {
		for(x = 0; x < request->files->nelts ; x++) {
			if(stream_content(r, cfg, banks[x], request) != OK)
				return OK;
		}
	}

	return OK;
}

/*Yes there is a reason to leave this here,
	its handy for someone who just wants to
	do a sethandler on a directry and walk
	away from it.
*/
static int mp3_random_handler(request_rec *r) {
	int x = 0;
	mp3_data **banks = NULL;
	mp3_conf *cfg = ap_get_module_config (r->per_dir_config, &mp3_module);
	request_data *request = ap_get_module_config(r->request_config, &mp3_module);
	/* for better PRNG seeding with microseconds versus current seconds */
	struct timeval tv_struct[2];
	gettimeofday(tv_struct, 0);
	srandom (tv_struct[0].tv_sec % tv_struct[0].tv_usec);


#ifdef DEBUG
	table_list("HEADERS ", r->headers_in);
#endif

	banks = (mp3_data **)cfg->files->elts;

	send_headers(r, cfg, request);

	for(;;) {
		x = ((int)random() % cfg->files->nelts);
		if(stream_content(r, cfg, banks[x], request) != OK)
			return OK;
	}

	return OK;
}

static int mp3_stream_handler(request_rec *r) {
	mp3_conf *cfg = ap_get_module_config (r->per_dir_config, &mp3_module);
	request_data *request = ap_get_module_config(r->request_config, &mp3_module);
	FILE *file;
	int b = 0;

#ifdef DEBUG
	printf("Playing stream %s\n", cfg->local_stream);
#endif
	if (!(file = ap_pfopen(r->pool, cfg->local_stream, "r"))) { 
		ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r, "Could not open file: %s(%s)", cfg->local_stream, strerror(errno));
		return HTTP_NOT_FOUND;
	}
	while((b = fgetc(file)) != EOF){
		if(ap_rputc(b,r) == -1)
			return HTTP_REQUEST_TIME_OUT;
	}

	return OK;
}

static const command_rec mp3_module_cmds[] = {
	{"MP3Engine", ap_set_flag_slot, 
		(void *) XtOffsetOf(mp3_conf, enabled), OR_ALL, FLAG,
		"Use this to turn on and off the mp3 steaming server."},
	{"MP3Random", ap_set_flag_slot, 
		(void *) XtOffsetOf(mp3_conf, random_enabled), OR_ALL, FLAG,
		"Use this to turn on and off random mp3."},
	{"MP3Cache", enable_cache, NULL, OR_ALL, FLAG,
		"Use this to turn on and off caching mp3 (you better have some memory)."},
	{"MP3", add_mp3, NULL, OR_ALL, TAKE1,
		"Takes either a filename, or a directory name to read files from."},
	{"MP3PlayList", add_mp3_playlist, NULL, OR_ALL, TAKE1,
		"Takes a file and loads each line into the server as a file (this is the standard file format for playlists)."},
	{"MP3LocalStream", add_local_stream, NULL, OR_ALL, TAKE1,
		"Takes a single parameter, the name of the local stream ."},
	{"MP3Genre", ap_set_string_slot, (void *) XtOffsetOf(mp3_conf, genre_name), OR_ALL, TAKE1, 
		"What genre the music is."},
	{"MP3CastName", ap_set_string_slot, (void *) XtOffsetOf(mp3_conf, cast_name), OR_ALL, TAKE1, 
		"What is the name of the feed."},
	{"MP3ReloadRequest", enable_request_reload, NULL, OR_ALL, FLAG,
		"This takes no value, it will cause the server to reload all directories/files/playlists for each connection. Be careful about using this since it can really kill performance. Cache can not be enabled while using this."},
	{NULL}
};

/* Dispatch list of content handlers */
static const handler_rec mp3_handlers[] = { 
    { "mp3", mp3_handler }, 
    { "mp3-random", mp3_random_handler }, 
    { "mp3-selection", mp3_selection_handler }, 
    { "mp3-rss", mp3_rss_handler }, 
    { "mp3-stream", mp3_stream_handler }, 
    { NULL, NULL }
};

static void mp3_init(server_rec * s, pool * p) {
	/* Tell apache we're here */
	ap_add_version_component("mod_mp3/"VERSION);
}

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT mp3_module = {
	STANDARD_MODULE_STUFF, 
	mp3_init,              /* module initializer                  */
	mconfig_for_directory, /* create per-dir    config structures */
	NULL,                  /* merge  per-dir    config structures */
	NULL,                  /* create per-server config structures */
	NULL,                  /* merge  per-server config structures */
	mp3_module_cmds,       /* table of config file commands       */
	mp3_handlers,          /* [#8] MIME-typed-dispatched handlers */
	NULL,                  /* [#1] URI to filename translation    */
	NULL,                  /* [#4] validate user id from request  */
	NULL,                  /* [#5] check if the user is ok _here_ */
	NULL,                  /* [#3] check access by host address   */
	NULL,                  /* [#6] determine MIME type            */
	mp3_fixup,             /* [#7] pre-run fixups                 */
	NULL,                  /* [#9] log a transaction              */
	NULL,                  /* [#2] header parser                  */
	NULL,                  /* child_init                          */
	NULL,                  /* child_exit                          */
	NULL                   /* [#0] post read-request              */
#ifdef EAPI
	,NULL,                  /* EAPI: add_module                    */
	NULL,                  /* EAPI: remove_module                 */
	NULL,                  /* EAPI: rewrite_command               */
	NULL                   /* EAPI: new_connection                */
#endif
};
