/* 
**  mod_mp3.h
*/ 

#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include "http_log.h"
#include "fnmatch.h"
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include "directives.h"
#include "version.h"

#define CORE_PRIVATE

#define WATCHPOINT printf("WATCHPOINT %s %d\n", __FILE__, __LINE__);

#define MP3_EXPORT(type) type

module MODULE_VAR_EXPORT mp3_module;
#define DEFAULT_CAST_NAME "Apache/mod_mp3 cast"
#define DEFAULT_GENRE_NAME "misc"


#define BITRATE 128
#define NUM_GENRE 148

#define TIME_FORMAT "%a %b %d %H:%M:%S %Y"
/* This is the XML header file */
#define HEADER "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n<!DOCTYPE rss PUBLIC \"-//Netscape Communications//DTD RSS 0.91//EN\"\n\"http://www.scripting.com/dtd/rss-0_91.dtd\">\n\n<rss version=\"0.91\">\n\n<channel>"

#define FOOTER "</channel>\n</rss>"

typedef struct {
	int enabled;  											/* If the mp3 engine is enabled */
	int random_enabled;									/* If the playlist should be random */
	int cache_enabled;									/* If cache is enabled */
	int reload_request;									/* This causes reloads to occr every request */
	int shmid;													/* SHM id to the scoreboard  */
	time_t generation;									/* Last time the files were reloaded  */
	char *local_stream;									/* If local_streams have been enabled (and the name of it) */
	array_header *files;								/* See mp3 data, this is the list */
	const char *cast_name;							/* Name of the cast */
	const char *genre_name;							/* Genre name for the music */
	array_header *load_files;						/* List of files to be used to load content */
	array_header *load_directories;			/* List of directories to be used if reload is called */
	array_header *load_playlists;				/* Playlists to reload */
} mp3_conf;														/* This the info that is stored on each section (directory, location, virtualhost...) */

typedef struct {
   char tag[3];
   char songname[30];
   char artist[30];
   char album[30];
   char year[4];
	 char comment[30];
   unsigned char genre;
}id3;

typedef struct {
	off_t size;							/* Size of a file */
	const char *name;				/* Name of the file */
	const char *filename;		/* Filename name */
	const char *signature;	/* MD5 signature of the filename */
	time_t time;						/* Time that the file was last modified */
	id3 id;								/* ID3 Tag */
	void *data;							/* The file (if cache has been enabled */
} mp3_data;								/* This is where we store file info */

typedef struct {
	table *args; 					/* Parsed query args */
	const char *op;				/* Current operation */
	const char *command;	/* Current command */
	const char *order;		/* Current order to play in */
	int random;						/* Should the music be played in a random order */
	const char *url;			/* URL for the stream */
	int shout; 						/* Stream via Shout protocol */
	int udp; 							/* Send udp stream */
	array_header *files;	/* Songs to play */
	int type;							/* This means:
													-1 means HTTP headers
													0 means shout headers
													1 use ICY headers
												*/
} request_data;					/* This is created for every connection */

typedef struct {
	time_t time;				/* Time of last regeneration */
} scoreboard;				/* This is shared memory stuff for each instance */

#include "proto.h"
